package jp.gr.java_conf.skrb.game.trivia.client;

import java.applet.AudioClip;
import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.FileInputStream;
import java.util.Collections;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.GrayFilter;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;

import jp.gr.java_conf.skrb.game.trivia.util.Counter;
import jp.gr.java_conf.skrb.game.trivia.util.CounterEvent;
import jp.gr.java_conf.skrb.game.trivia.util.CounterListener;
import jp.gr.java_conf.skrb.game.trivia.util.CounterPanel;

public class ClientPanel extends JPanel {
    private AudioClip clip;
    private ImageIcon normalIcon;
    private ImageIcon pushedIcon;
    private ImageIcon disableIcon;
    private JLabel button;
    private MouseListener mouseListener;
    
    private Counter counter;
    private CounterPanel counterPanel;

    private Set<CounterListener> listeners;
    private boolean isEnabled;

    public static final String AUDIO = "AudioClip";
    public static final String BASE = "BaseImage";
    public static final String NORMAL = "NormalImage";
    public static final String PUSHED = "PushedImage";
    public static final String COUNTER_LOCATION_X = "CounterLocationX";
    public static final String COUNTER_LOCATION_Y = "CounterLocationY";

    public ClientPanel(Counter counter,  Properties properties) {
        listeners = Collections.synchronizedSet(new HashSet<CounterListener>());

        this.counter = counter;

        setLayout(new BorderLayout());

        JLayeredPane pane = new JLayeredPane();

        initAudioClip(properties);
        initBase(pane, properties);
        initButton(pane, properties);
        initCounterPanel(pane, properties, counter);

        add(pane, BorderLayout.CENTER);
    }


    private void initAudioClip(Properties properties) {
        try {
            String audioclip = properties.getProperty(AUDIO);
            FileInputStream in = new FileInputStream(audioclip);
            clip = new com.sun.media.sound.JavaSoundAudioClip(in);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void initBase(JLayeredPane pane, Properties properties) {
        String baseimage = properties.getProperty(BASE);
        JLabel base = new JLabel(new ImageIcon(baseimage));
        base.setBounds(0, 0, base.getPreferredSize().width, base.getPreferredSize().height);

        pane.setPreferredSize(base.getPreferredSize());
        pane.add(base, new Integer(0));
    }

    private void initButton(JLayeredPane pane, Properties properties) {
        String normalimage = properties.getProperty(NORMAL);
        String pushedimage = properties.getProperty(PUSHED);

        normalIcon = new ImageIcon(normalimage);
        pushedIcon = new ImageIcon(pushedimage);
        disableIcon = new ImageIcon(GrayFilter.createDisabledImage(normalIcon.getImage()));

        button = new JLabel(disableIcon);
        
        button.setBounds(0, 0, button.getPreferredSize().width, button.getPreferredSize().height);
        pane.add(button, new Integer(1));

        mouseListener = new MouseAdapter() {
                public void mousePressed(MouseEvent e) {
                    button.setIcon(pushedIcon);

                    counter.increment();
                    if (counter.isValid()) {
                        clip.play();
                        fireCounterIncrementedEvent();
                    }

                    counterPanel.repaint();
                }
                
                public void mouseReleased(MouseEvent e) {
                    button.setIcon(normalIcon);
                }
            };
    }

    private void initCounterPanel(JLayeredPane pane, Properties properties, Counter counter) {
        int locationX = Integer.parseInt(properties.getProperty(COUNTER_LOCATION_X));
        int locationY = Integer.parseInt(properties.getProperty(COUNTER_LOCATION_Y));

        counterPanel = createCounterPanel(properties, counter);
        counterPanel.setBounds(locationX, locationY,
                               counterPanel.getPreferredSize().width,
                               counterPanel.getPreferredSize().height);
        pane.add(counterPanel, new Integer(2));
    }

    private CounterPanel createCounterPanel(Properties properties, Counter counter) {
        return new CounterPanel(properties, counter);
    }

    public void setEnabled(boolean flag) {
        if (flag) {
            if (!isEnabled) {
                button.addMouseListener(mouseListener);
            }

            button.setIcon(normalIcon);
        } else {
            button.removeMouseListener(mouseListener);
            button.setIcon(disableIcon);
        }

        isEnabled = flag;
    }

    private void fireCounterIncrementedEvent() {
        CounterEvent event = new CounterEvent(counter);
        
        for (CounterListener listener: listeners) {
            listener.counterIncremented(event);
        }
    }

    public void addCounterListener(CounterListener listener) {
        listeners.add(listener);
    }

    public void removeMouseListener(MouseListener listener) {
        listeners.remove(listener);
    }
}
