package jp.gr.java_conf.skrb.game.trivia.server;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import javax.swing.JButton;
import javax.swing.JFrame;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;

import jp.gr.java_conf.skrb.game.trivia.util.Counter;
import jp.gr.java_conf.skrb.game.trivia.util.CounterPanel;


public class DefaultServerView implements ServerView {
    private TriviaServer server;
    private TotalCounter totalCounter;
    private Map<Integer, ServerPanel> clients;
    private Properties properties;

    private JFrame frame;
    private JLabel totalCounterLabel;
    private JPanel container;
    private JFrame waitingListFrame;
    private JList waitingList;

    public DefaultServerView(TriviaServer server, TotalCounter counter, 
                             Collection waitingClients, Properties properties) {
        this.server = server;
        this.totalCounter = counter;
        this.properties = properties;
        
        clients = new HashMap<Integer, ServerPanel>();
        
        initView(waitingClients);
    }

    private void initView(Collection waitingClients) {
        frame = new JFrame("TriviaServer");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setLocationByPlatform(true);

        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        totalCounterLabel = new JLabel("000/000", JLabel.CENTER);
        Font currentFont = totalCounterLabel.getFont();
        totalCounterLabel.setFont(new Font(currentFont.getName(), 
                                           currentFont.getStyle(),
                                           32));
        panel.add(totalCounterLabel, BorderLayout.CENTER);

        JButton reset = new JButton("Reset");
        reset.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    server.reset();
                }
            });
        panel.add(reset, BorderLayout.EAST);
        frame.getContentPane().add(panel, BorderLayout.NORTH);

        container = new JPanel();
        container.setLayout(new FlowLayout());
        frame.getContentPane().add(container, BorderLayout.CENTER);
        frame.pack();
        frame.setVisible(true);

        waitingListFrame = new JFrame("Waiting List");
        waitingListFrame.setSize(100, 200);
        waitingListFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        waitingList = new JList(waitingClients.toArray());
        waitingListFrame.getContentPane().add(waitingList);

        Rectangle bounds = frame.getBounds();
        waitingListFrame.setLocation(bounds.x + bounds.width, bounds.y);
        waitingListFrame.setVisible(true);
    }

    private void reset() {
        updateTotalCount();

        for (ServerPanel panel : clients.values()) {
            panel.updateView();
        }
    }

    public void updateWaitingList(Collection waitingClients) {
        waitingList.setListData(waitingClients.toArray());
    }

    public void addClient(Integer id, String name, Counter counter) {
        ServerPanel panel = new ServerPanel(name, properties, counter);
        clients.put(id, panel);

        updateTotalCount();

        container.add(panel);
        frame.pack();

        Rectangle bounds = frame.getBounds();
        waitingListFrame.setLocation(bounds.x + bounds.width, bounds.y);
    }

    public void removeClient(Integer id) {
        ServerPanel panel = clients.remove(id);
        container.remove(panel);
        frame.pack();

        Rectangle bounds = frame.getBounds();
        waitingListFrame.setLocation(bounds.x + bounds.width, bounds.y);
    }

    private void updateTotalCount() {
        if (totalCounter.isMax() && totalCounter.getCount() != 0) {
            totalCounterLabel.setForeground(Color.RED);
        } else {
            totalCounterLabel.setForeground(Color.BLACK);
        }

        totalCounterLabel.setText(String.format("%03d/%03d", 
                                              totalCounter.getCount(),
                                              totalCounter.getMaxCount()));
    }

    public void updateClient(Integer id) {
        ServerPanel panel = clients.get(id);
        updateClient(panel);

        updateTotalCount();
    }

    private void updateClient(ServerPanel panel) {
        panel.repaint();
    }

    public void resetClients() {
        for (ServerPanel panel: clients.values()) {
            updateClient(panel);
        }

        updateTotalCount();
    }
}
