package jp.gr.java_conf.skrb.game.trivia.server;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

public class ServerCommunicator {
    private static final int PORT = 9000;

    private TriviaServer server;
    private ServerSocket serverSocket;

    private Thread acceptThread;

    public ServerCommunicator(TriviaServer server) throws IOException {
        this.server = server;
    }

    public void accept() throws IOException {
        serverSocket = new ServerSocket(PORT);

        acceptThread = new Thread(new Runnable() {
                public void run() {
                    Thread current = Thread.currentThread();

                    try {
                        while (acceptThread == current) {
                            Socket socket = serverSocket.accept();
                            new Thread(new Reader(socket)).start();
                        }
                    } catch (IOException ex) {
                        ex.printStackTrace();
                        System.exit(1);
                    }
                }
            });
        acceptThread.start();
    }

    class Reader implements Runnable {
        private Socket socket;
        private DataInputStream istream;

        public Reader(Socket socket) {
            this.socket = socket;
            try {
                istream = new DataInputStream(socket.getInputStream());
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }

        public void run() {
            try {
                String name = istream.readUTF();
                server.connectClient(socket, name);

                while (true) {
                    int count = istream.readInt();
                    server.readData(socket, count);
                }
            } catch (IOException ex) {
                server.disconnectClient(socket);
                try {
                    socket.close();
                } catch (IOException ex2) {}
            }
        }
    }

    
    public void sendReset(Socket socket) {
        try {
            DataOutputStream ostream = new DataOutputStream(socket.getOutputStream());
            ostream.writeInt(-1);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    public void close(Socket socket) {
        try {
            socket.close();
        } catch(IOException ex) {
            ex.printStackTrace();
            System.exit(1);
        }
    }

    public void closeServerSocket() {
        try {
            serverSocket.close();
        } catch(IOException ex) {
            ex.printStackTrace();
            System.exit(1);
        }
    }
}
