package jp.gr.java_conf.skrb.game.trivia.server;

import java.applet.AudioClip;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;

import jp.gr.java_conf.skrb.game.trivia.util.Counter;
import jp.gr.java_conf.skrb.game.trivia.util.CounterEvent;
import jp.gr.java_conf.skrb.game.trivia.util.CounterListener;
import jp.gr.java_conf.skrb.game.trivia.util.CounterPanel;

public class ServerPanel extends JPanel {
    private AudioClip clip;
    
    private Counter counter;
    private CounterPanel counterPanel;

    public static final String AUDIO = "AudioClip";
    public static final String BASE = "BaseImage";

    public ServerPanel(String name, Properties properties, Counter counter) {
        setLayout(new BorderLayout());

        JLabel label = new JLabel(name, JLabel.CENTER);
        Font currentFont = label.getFont();
        label.setFont(new Font(currentFont.getName(), 
                               currentFont.getStyle(),
                               currentFont.getSize() + 16));
        add(label, BorderLayout.NORTH);

        this.counter = counter;

        JLayeredPane pane = new JLayeredPane();

        initAudioClip(properties);
        initBase(pane, properties);
        initCounterPanel(pane, properties, counter);

        add(pane, BorderLayout.CENTER);
    }

    private void initAudioClip(Properties properties) {
        try {
            String audioclip = properties.getProperty(AUDIO);
            FileInputStream in = new FileInputStream(audioclip);
            clip = new com.sun.media.sound.JavaSoundAudioClip(in);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void initBase(JLayeredPane pane, Properties properties) {
        String baseimage = properties.getProperty(BASE);
        JLabel base = new JLabel(new ImageIcon(baseimage));
        base.setBounds(0, 0, base.getPreferredSize().width, base.getPreferredSize().height);

        pane.setPreferredSize(base.getPreferredSize());
        pane.add(base, new Integer(0));
    }

    private void initCounterPanel(JLayeredPane pane, Properties properties, Counter counter) {
        counterPanel = createCounterPanel(properties, counter);
        counterPanel.setBounds(0, 0,
                               counterPanel.getPreferredSize().width,
                               counterPanel.getPreferredSize().height);
        pane.add(counterPanel, new Integer(2));
    }

    private CounterPanel createCounterPanel(Properties properties, Counter counter) {
        return new CounterPanel(properties, counter);
    }

    public void updateView() {
        if (counter.isValid()) {
            clip.play();
        }

        repaint();
    }

    public void setCount(int count) {
        counter.setCount(count);
        
//        if (counter.isValid()) {
//            clip.play();
//        }

        repaint();
    }

    public void resetCount() {
        counter.reset();
        repaint();
    }
}
